#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wily', 'wily.archivers', 'wily.commands', 'wily.helper', 'wily.operators']

package_data = \
{'': ['*'],
 'wily': ['locales/*',
          'locales/de/*',
          'locales/de/LC_MESSAGES/*',
          'locales/en/*',
          'locales/en/LC_MESSAGES/*',
          'locales/en_AU/*',
          'locales/en_AU/LC_MESSAGES/*',
          'locales/ja/*',
          'locales/ja/LC_MESSAGES/*',
          'templates/*',
          'templates/css/*']}

package_dir = \
{'': 'src'}

install_requires = \
['gitpython>=3.0.0,<4.0.0',
 'radon>=5.1,<5.2',
 'click>=7.0,<9.0',
 'nbformat>=5.1.3,<6.0.0',
 'colorlog>=4.0.0,<5.0.0',
 'tabulate>=0.8.2,<1.0.0',
 'plotly>=4.0.0,<6.0.0',
 'progress>=1.4,<2.0']

extras_require = \
{":python_version == '3.6'": ['dataclasses'],
 'dev': ['bandit~=1.7.4',
         'black~=22.6.0',
         'codespell~=2.1.0',
         'flake8~=4.0.1',
         'flake8-bugbear~=22.7.1',
         'flake8-comprehensions~=3.10.0',
         'flake8-2020~=1.6.1',
         'isort~=5.10.1',
         'mypy~=0.961',
         'pydocstyle~=6.1.1',
         'pyupgrade~=2.37.1',
         'safety~=2.3.4'],
 'test': ['pytest~=7.2', 'pytest-cov~=3.0.0']}

entry_points = \
{'console_scripts': ['wily = wily.__main__:cli']}

setup(name='wily',
      version='1.24.2',
      description='A Python application for tracking, reporting on timing and complexity in tests and applications.',
      author=None,
      author_email='Anthony Shaw <anthonyshaw@apache.org>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
